"use strict";
var express = require("express");
var bodyParser = require("body-parser");
var cookieParser = require("cookie-parser");
var favicon = require("serve-favicon");
var logger = require("morgan");
var winston = require("winston");
var graphqlHTTP = require("express-graphql");
var fs = require("fs");
var https = require("https");
var app = express();
app.disable('x-powered-by');
var port = process.env.PORT || 8001;
var environment = process.env.NODE_ENV;
app.use(bodyParser.urlencoded({ extended: true }));
app.use(bodyParser.json());
app.use(cookieParser());
app.use(logger('dev'));
var LOG_DIR = 'logs/';
if (!fs.existsSync(LOG_DIR)) {
    winston.log('info', 'Create logs directory');
    fs.mkdirSync(LOG_DIR);
}
winston.configure({
    level: 'debug',
    transports: [
        new (winston.transports.Console)(),
        new (winston.transports.File)({ filename: LOG_DIR + 'actiondesigner.log' })
    ]
});
var _1 = require("./schema/");
app.use('/graphql', graphqlHTTP({
    schema: _1.default,
    graphiql: true,
}));
winston.log('info', 'PORT=' + port);
winston.log('info', 'NODE_ENV=' + environment);
switch (environment) {
    case 'build':
        // THIS MUST BE REMOVED BEFORE RELEASE
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
        winston.log('info', '** BUILD **');
        app.use(favicon(__dirname + '/build/favicon.ico'));
        app.use(express.static(__dirname + '/build'));
        // Any deep link calls should return index.html
        app.get('*', function (req, res) {
            res.sendFile(__dirname + '/build/index.html');
        });
        break;
    default:
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
        winston.log('info', '** DEV **');
        app.use(express.static('./dist/client/'));
        app.use(express.static('./'));
        app.use(express.static('./tmp'));
        break;
}
https.createServer({
    key: fs.readFileSync('./ssl/server.key'),
    cert: fs.readFileSync('./ssl/server.crt'),
    ca: fs.readFileSync('./ssl/ca.crt'),
    requestCert: true,
    rejectUnauthorized: false
}, app).listen(port, function () {
    winston.log('info', 'Express server listening on port ' + port);
    winston.log('info', 'env = ' + app.get('env') +
        '\n__dirname = ' + __dirname +
        '\nprocess.cwd = ' + process.cwd());
});
