"use strict";
var request = require("request");
var graphql_1 = require("graphql");
var config_1 = require("./../../config");
var action_type_1 = require("./../types/action.type");
var parameter_type_1 = require("./../types/parameter.type");
var ActionCreateInputType = new graphql_1.GraphQLInputObjectType({
    name: 'ActionCreateInput',
    fields: {
        name: { type: graphql_1.GraphQLString },
        parameters: { type: parameter_type_1.default },
        tags: { type: new graphql_1.GraphQLList(graphql_1.GraphQLString) },
        description: { type: graphql_1.GraphQLString },
        enabled: { type: graphql_1.GraphQLBoolean },
        entry_point: { type: graphql_1.GraphQLString },
        uid: { type: graphql_1.GraphQLString },
        pack: { type: graphql_1.GraphQLString },
        ref: { type: graphql_1.GraphQLString },
        runner_type: { type: graphql_1.GraphQLString },
        data_files: {
            type: new graphql_1.GraphQLList(new graphql_1.GraphQLInputObjectType({
                name: 'DataFileInput',
                fields: {
                    content: { type: graphql_1.GraphQLString },
                    file_path: { type: graphql_1.GraphQLString }
                }
            }))
        }
    }
});
var CreateActionMutation = {
    type: action_type_1.default,
    args: {
        token: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString) },
        action: { type: new graphql_1.GraphQLNonNull(ActionCreateInputType) }
    },
    resolve: function (obj, _a, context) {
        var token = _a.token, action = _a.action;
        return new Promise(function (fulfull, reject) {
            request.post(config_1.default.workflowHost + "/api/v1/actions", {
                json: action,
                headers: {
                    'X-Auth-Token': token
                }
            }, function (error, response, body) {
                // validation of response
                if (error)
                    return reject(error);
                if (body && body.faultstring)
                    return reject(body.faultstring);
                fulfull(body);
            });
        });
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = CreateActionMutation;
