"use strict";
var request = require("request");
var graphql_1 = require("graphql");
var config_1 = require("./../../config");
var action_type_1 = require("./../types/action.type");
var notify_type_1 = require("./../types/notify.type");
var parameter_type_1 = require("./../types/parameter.type");
var ActionEditInputType = new graphql_1.GraphQLInputObjectType({
    name: 'ActionEditInput',
    fields: {
        name: { type: graphql_1.GraphQLString },
        parameters: { type: parameter_type_1.default },
        tags: { type: new graphql_1.GraphQLList(graphql_1.GraphQLString) },
        description: { type: graphql_1.GraphQLString },
        enabled: { type: graphql_1.GraphQLBoolean },
        entry_point: { type: graphql_1.GraphQLString },
        notify: { type: notify_type_1.default },
        uid: { type: graphql_1.GraphQLString },
        pack: { type: graphql_1.GraphQLString },
        ref: { type: graphql_1.GraphQLString },
        id: { type: graphql_1.GraphQLString },
        runner_type: { type: graphql_1.GraphQLString }
    }
});
var EditActionMutation = {
    type: action_type_1.default,
    args: {
        token: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString) },
        action: { type: new graphql_1.GraphQLNonNull(ActionEditInputType) },
        code: { type: graphql_1.GraphQLString }
    },
    resolve: function (obj, _a, context) {
        var token = _a.token, action = _a.action, code = _a.code;
        // the remote API expects code to be inside of a data_files object
        if (code) {
            action.data_files = [{
                    content: code,
                    file_path: action.entry_point
                }];
        }
        return new Promise(function (fulfull, reject) {
            request.put(config_1.default.workflowHost + ("/api/v1/actions/" + action.id), {
                json: action,
                headers: {
                    'X-Auth-Token': token
                }
            }, function (error, response, body) {
                // validation of response
                if (error)
                    return reject(error);
                if (body && body.faultstring)
                    return reject(body.faultstring);
                fulfull(body);
            });
        });
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = EditActionMutation;
