"use strict";
var request = require("request");
var graphql_1 = require("graphql");
var config_1 = require("./../../../config");
var pack_type_1 = require("./../../types/pack.type");
var GetPacksWithActionsQuery = {
    type: new graphql_1.GraphQLList(pack_type_1.default),
    description: 'The Packs that are defined in the workflow system.',
    args: {
        token: { type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString) }
    },
    resolve: getPacks
};
function getPacks(obj, _a, context) {
    var token = _a.token;
    return new Promise(function (fulfill, reject) {
        request.get(config_1.default.workflowHost + "/api/v1/packs", {
            headers: {
                'X-Auth-Token': token
            },
            json: true
        }, function (error, response, body) {
            // validation response
            if (error)
                return reject(error);
            if (body && body.faultstring)
                return reject(body.faultstring);
            var packs = body;
            // get actions for each of the packs
            getActions(token).then(function (actions) {
                packs = addActionsToPacks(packs, actions);
                fulfill(packs);
            }).catch(function (error) {
                reject(error);
            });
        });
    });
}
function addActionsToPacks(packs, actions) {
    packs.forEach(function (pack, index) {
        pack.actions = actions.filter(function (action) {
            if (action.pack === pack.name && action.runner_type === 'mistral-v2')
                return action;
        });
        if (pack.actions.length === 0) {
            delete packs[index];
        }
    });
    return packs;
}
function getActions(token) {
    return new Promise(function (fulfill, reject) {
        request.get(config_1.default.workflowHost + '/api/v1/actions?exclude_attributes=parameters%2Cnotify', {
            headers: {
                'X-Auth-Token': token
            },
            json: true
        }, function (error, response, body) {
            // validation response
            if (error)
                return reject(error);
            if (body && body.faultstring)
                return reject(body.faultstring);
            fulfill(body);
        });
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = GetPacksWithActionsQuery;
